part of play;

class PlayVideoPlayer {
  late dart_vlc.Player desktopPlayer;
  late video_player.VideoPlayerController mobilePlayer;
  PlayVideoPlayer();

  Future<void> initialize() async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer = dart_vlc.Player(id: 0, videoDimensions: const dart_vlc.VideoDimensions(640, 460), registerTexture: !Platform.isWindows);
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = video_player.VideoPlayerController.network("https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4");
      await mobilePlayer.initialize();
    }
  }

  test() async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(
              File('/home/hexaminate/Downloads/y2mate.com - NMIXX OO Performance Video_1080p.mp4'),
            ),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = video_player.VideoPlayerController.network("https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4");
      await mobilePlayer.initialize();
    }
  }

  void playFromFile(File file) {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(file),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    }
  }

  FutureOr openFile(File file) async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(file),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = video_player.VideoPlayerController.file(file);
      await mobilePlayer.initialize();
    }
  }

  Duration getDurationMax() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.position.duration ?? Duration();
    }
    return Duration();
  }

  Duration getDurationPosition() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.position.position ?? Duration();
    }
    return Duration();
  }

  streamDurationPosition() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.positionStream;
    } else if (Platform.isAndroid || Platform.isIOS) {}
    return Stream.value("ok");
  }

  seek(Duration duration) {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.seek(duration);
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer.seekTo(duration);
    }
  }

  playOrPause() {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.playOrPause();
    }
    if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer.value.isPlaying ? mobilePlayer.pause() : mobilePlayer.play();
    }
  }
}

class PlayVideoWidget extends StatefulWidget {
  final double height;
  final double width;
  final double videoHeight;
  final double videoWidth;
  final Duration videoDurationMax;
  final Duration videoDurationPosition;
  final void Function(double newValue)? onChangedDuration;
  PlayVideoPlayer playVideoPlayer;
  PlayVideoWidget({
    Key? key,
    required this.playVideoPlayer,
    required this.height,
    required this.width,
    required this.videoHeight,
    required this.videoWidth,
    required this.videoDurationMax,
    required this.videoDurationPosition,
    required this.onChangedDuration,
  }) : super(key: key);

  @override
  State<PlayVideoWidget> createState() => _PlayVideoWidgetState();
}

class _PlayVideoWidgetState extends State<PlayVideoWidget> {
  @override
  void initState() {
    super.initState();
    taskVideo();
  }

  taskVideo() async {
    await widget.playVideoPlayer.initialize();
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    late Widget videoWidget = Container();
    if (Platform.isWindows || Platform.isLinux) {
      videoWidget = dart_vlc.Video(
        player: widget.playVideoPlayer.desktopPlayer,
        width: widget.videoHeight,
        height: widget.videoWidth,
        volumeThumbColor: Colors.blue,
        volumeActiveColor: Colors.blue,
        showControls: false,
        showFullscreenButton: true,
        showTimeLeft: true,
        fillColor: Colors.black,
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      videoWidget = video_player.VideoPlayer(widget.playVideoPlayer.mobilePlayer);
    }

    return Container(
      height: widget.height,
      width: widget.width,
      decoration: BoxDecoration(
        color: Colors.black,
        borderRadius: BorderRadius.all(
          Radius.circular(20),
        ),
      ),
      child: Stack(
        fit: StackFit.passthrough,
        children: [
          videoWidget,
          Positioned(
            bottom: 5,
            left: 0,
            right: 0,
            child: Padding(
              padding: const EdgeInsets.symmetric(horizontal: 20),
              child: Row(
                children: [
                  InkWell(
                    onTap: () {
                      widget.playVideoPlayer.playOrPause();
                      setState(() {});
                    },
                    child: const Icon(
                      Icons.pause,
                      color: Colors.white,
                    ),
                  ),
                  InkWell(
                    onTap: () {},
                    child: const Icon(
                      Icons.skip_next,
                      color: Colors.white,
                    ),
                  ),
                  Expanded(
                    child: StreamBuilder(
                      stream: widget.playVideoPlayer.streamDurationPosition(),
                      builder: (BuildContext context, AsyncSnapshot snapshot) {
                        return Slider(
                          min: 0,
                          max: widget.playVideoPlayer.getDurationMax().inMilliseconds.toDouble(),
                          value: widget.playVideoPlayer.getDurationPosition().inMilliseconds.toDouble(),
                          onChanged: (double value) {
                            setState(() {
                              widget.playVideoPlayer.seek(Duration(milliseconds: value.toInt()));
                            });
                          },
                        );
                      },
                    ),
                  ),
                  InkWell(
                    onTap: () {},
                    child: const Icon(
                      Icons.fullscreen,
                      color: Colors.white,
                    ),
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}
